use core::time::Duration;
use std::collections::HashMap;

use bitfinex::api::Bitfinex;
use bitfinex::positions::Position;
use bitfinex::ticker::TradingPairTicker;
use tokio::time::delay_for;

use crate::connectors::Connector;
use crate::currency::{Symbol, SymbolPair};
use crate::pairs::PairStatus;
use crate::ticker::Ticker;
use crate::BoxError;

pub struct BfxBot<'a> {
    connector: Box<dyn Connector + 'a>,
    ticker: Ticker,
    pair_status: Vec<PairStatus>,
    quote: Symbol,
    trading_symbols: Vec<Symbol>,
    // account_info: String,
    // ledger: String,
}

impl<'a> BfxBot<'a> {
    pub fn new<C: Connector + 'a>(
        connector: C,
        trading_symbols: Vec<Symbol>,
        quote: Symbol,
        tick_duration: Duration,
    ) -> Self {
        BfxBot {
            connector: Box::new(connector),
            ticker: Ticker::new(tick_duration),
            pair_status: trading_symbols
                .iter()
                .map(|x| SymbolPair::new(quote.clone(), x.clone()))
                .map(|x| PairStatus::new(x, 1, None))
                .collect(),
            quote,
            // account_info: String::new(),
            // ledger: String::new(),
            trading_symbols,
        }
    }

    pub async fn current_prices(&self, symbol: Symbol) -> Result<TradingPairTicker, BoxError> {
        let trading_pair = SymbolPair::new(self.quote.clone(), symbol.clone());

        if !self.trading_symbols.contains(&symbol) {
            return Err("Symbol not supported.".into());
        }

        self.connector.current_prices(&trading_pair).await
    }

    pub async fn update(&mut self) {
        println!("Updating...");
        delay_for(self.ticker.duration()).await;
        self.ticker.inc();
        // self.update_pairs().await;
        println!("Done!");
    }

    // async fn update_pairs(&mut self) {
    //     let active_positions = self.bfx.positions.active_positions().await?;
    //
    //     for p in active_positions {}
    // }
}
